#!/usr/bin/perl -w
# gencgi.pl
# Generate our CGI parsing functions
# $Fuller: myprograms/gencgi/gencgi.pl,v 1.3 2003/05/15 21:25:25 fullermd Exp $

#use strict;

# Our first arg should be a file containing the hash of what variables
# and types we're going to be parsing.
if(!($ARGV[0]))
{
	print "Error: Need hashfile to use\n";
	exit(1);
}

if (!(-r $ARGV[0]))
{
	print "Error: Can't open file '$ARGV[0]' to read hash.\n";
	exit(1);
}

require "$ARGV[0]";

# First, start outputting the header file
open(HDRF, ">cgi_getdata.h") or die "Can't open cgi_getdata.h";
print("Writing header file 'cgi_getdata.h'......  ");
print HDRF <<_EOF_;
/* Header info */

/* Structure */
struct cgi_dat
{
_EOF_

foreach(sort(keys %CGIVARS))
{
	my $ename = $_;
	my $etype = $CGIVARS{"$_"};
	my $otype;
	if($etype eq "string")
	{ $otype = "char *"; }
	elsif($etype eq "num")
	{ $otype = "unsigned long"; }

	printf HDRF "\t%16s\t%s;\n", $otype, $ename;
}

print HDRF <<_EOF_;
};
typedef struct cgi_dat CGI_DAT;

/* Prototype */
CGI_DAT *cgi_getdata(void);

/* Constants */
#define VAR_STR		0
#define VAR_UL		1

_EOF_

close(HDRF);
print(" done.\n");


# Now, the parsing function itself
open(SRCF, ">cgi_getdata.c") or die "Can't open cgi_getdata.c";
print("Writing source file 'cgi_getdata.c'......  ");
print SRCF <<_EOF_;
/* CGI GET parsing function */
#include <stdlib.h>
#include <string.h>

/* Need "-I." on cc(1) command line for this */
#include <cgi_getdata.h>

CGI_DAT *
cgi_getdata(void)
{
	/* XXX malloc() failure can't be caught before we segfault */
	CGI_DAT			*toret = malloc(sizeof(CGI_DAT));
	char			*qstr, *buf, *buf2, *buf3;
	int				i;
	struct
	{
		char			*name;
		int				vtype;
		union
		{
			void			*dummy;
			char			**str;
			unsigned long	*ul;
		} cnt;
	} flds[] =
	{
_EOF_

# Slap in all the fields we're using
foreach(sort(keys %CGIVARS))
{
	my $ename = $_;
	my $etype = $CGIVARS{"$_"};
	my $otype;
	if($etype eq "string")
	{ $otype = "VAR_STR"; }
	elsif($etype eq "num")
	{ $otype = "VAR_UL"; }

	printf SRCF "\t\t{\"%s\", %s, {(&(toret->%s))} },\n", $ename, $otype, $ename;
}

# The rest is constant
print SRCF <<_EOF_;
		{NULL, 0, {NULL}}
	};


	if( (qstr=getenv("QUERY_STRING")) == NULL)
		return(NULL);

	for( buf = qstr ; (buf2 = strsep(&buf, "&")) != NULL ; /* */ )
	{
		if(*buf2 == '\\0')
			continue;

		if( (buf3 = strchr(buf2, '=')) == NULL)
			continue; /* Probably shouldn't happen */

		*buf3 = '\\0';
		buf3++;

		for(i=0 ; flds[i].name != NULL ; i++)
		{
			if(!(strcmp(flds[i].name, buf2)))
			{
				/* Match */
				if(flds[i].vtype == VAR_STR)
					*flds[i].cnt.str = strdup(buf3);
				else if(flds[i].vtype == VAR_UL)
					*flds[i].cnt.ul = strtoul(buf3, (char **)NULL, 10);
				break;
			}
		}
	}

	return(toret);
}
_EOF_

# And that's it.
close(SRCF);
print(" done.\n");

print("Done!\n");
